package tsgo

import (
	"errors"

	"github.com/godbus/dbus"
	"github.com/godbus/dbus/introspect"
)

var dbusConn *dbus.Conn

const intro = `
<node>
	<interface name="org.kapelle.ts3">
		<method name="ToggleMute"></method>
	</interface>` + introspect.IntrospectDataString + `
</node> `

type dbusHandler int

func (d dbusHandler) ToggleMute() *dbus.Error {
	Ts3Functions.PrintMessage(1, "toggle", PLUGIN_MESSAGE_TARGET_SERVER) // TODO placeholder
	return nil
}

func initDbus() error {
	var err error
	dbusConn, err = dbus.ConnectSessionBus()

	if err != nil {
		return err
	}

	err = dbusConn.Export(introspect.Introspectable(intro), "/", "org.freedesktop.DBus.Introspectable")
	if err != nil {
		return err
	}

	err = dbusConn.Export(dbusHandler(1), "/", "org.kapelle.ts3")
	if err != nil {
		return err
	}

	reply, err2 := dbusConn.RequestName("org.kapelle.ts3", dbus.NameFlagDoNotQueue)
	if err2 != nil {
		return err2
	}
	if reply != dbus.RequestNameReplyPrimaryOwner {
		return errors.New("name already taken")
	}

	return nil
}

func closeConn() {
	dbusConn.Close()
}

func notifyPoke(from, message string) {
	obj := dbusConn.Object("org.freedesktop.Notifications", "/org/freedesktop/Notifications")
	call := obj.Call("org.freedesktop.Notifications.Notify", 0, "teamspeak3", uint32(0),
		"", "Poke from "+from, message, []string{},
		map[string]dbus.Variant{"urgency": dbus.MakeVariant(byte(2))}, int32(-1))

	if call.Err != nil {
		// TODO error handleling
	}
}
